/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GFacRegistry.java,v 1.6 2007/01/22 23:49:37 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.gfac;

import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import xsul5.MLogger;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.gui.ComponentTreeNode;
import edu.indiana.extreme.xbaya.component.registry.ComponentRegistry;
import edu.indiana.extreme.xbaya.component.registry.ComponentRegistryException;
import edu.indiana.extreme.xbaya.component.ws.WSComponent;
import edu.indiana.extreme.xbaya.component.ws.WSComponentFactory;
import edu.indiana.extreme.xbaya.util.WSDLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class GFacRegistry extends ComponentRegistry {

    /**
     * Either abstract WSDL or concrete WSDL.
     */
    public enum Type {
        /**
         * ABSTRACT
         */
        ABSTRACT,

        /**
         * CONCRETE
         */
        CONCRETE;
    }

    private static final MLogger logger = MLogger.getLogger();

    private URI url;

    private Type type;

    private GFacRegistryClient client;

    /**
     * Constructs a GFacRegistry.
     * 
     * @param url
     * @param type
     */
    public GFacRegistry(URI url, Type type) {
        this.url = url;
        this.type = type;
        this.client = new GFacRegistryClient(WSDLUtil.appendWSDLQuary(url));
    }

    /**
     * @see edu.indiana.extreme.xbaya.component.registry.ComponentRegistry#getComponentTree()
     */
    @Override
    public ComponentTreeNode getComponentTree()
            throws ComponentRegistryException {

        ComponentTreeNode tree = new ComponentTreeNode(this);

        String[] qnameStrings;
        if (this.type == Type.CONCRETE) {
            // "" returns all CWSDLs.
            qnameStrings = this.client.findService("");
        } else {
            qnameStrings = this.client.findServiceDesc("");
        }

        Map<String, ComponentTreeNode> namespaceMap = new HashMap<String, ComponentTreeNode>();

        for (String qnameString : qnameStrings) {
            logger.finest("qnameString: " + qnameString);
            QName qname = QName.valueOf(qnameString);

            GFacComponentReference componentRef = new GFacComponentReference(
                    this, qname);
            String namespace = qname.getNamespaceURI();
            ComponentTreeNode namespaceNode = namespaceMap.get(namespace);
            if (namespaceNode == null) {
                namespaceNode = new ComponentTreeNode(namespace);
                namespaceMap.put(namespace, namespaceNode);
                tree.add(namespaceNode);
            }
            ComponentTreeNode treeLeaf = new ComponentTreeNode(componentRef);
            namespaceNode.add(treeLeaf);
        }
        return tree;
    }

    /**
     * @see edu.indiana.extreme.xbaya.component.registry.ComponentRegistry#getName()
     */
    @Override
    public String getName() {
        return this.url.toString();
    }

    /**
     * @param qname
     * @return The Component
     * @throws ComponentRegistryException
     * @throws ComponentException
     */
    public List<WSComponent> getComponents(QName qname)
            throws ComponentRegistryException, ComponentException {
        String wsdl;
        if (this.type == Type.CONCRETE) {
            wsdl = this.client.getConcreteWsdl(qname.toString());
        } else {
            wsdl = this.client.getAbstractWsdl(qname.toString());
        }
        logger.finest("concreateWSDL:" + wsdl);
        List<WSComponent> components = WSComponentFactory
                .createComponents(wsdl);
        return components;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
